package com.gic.haoban.manage.service.service.out.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.clerk.api.dto.ClerkQwDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.GlobalVar;
import com.gic.haoban.app.customer.service.api.service.InnerApiService;
import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.*;
import com.gic.haoban.manage.api.service.*;
import com.gic.haoban.manage.api.util.notify.NoticeMessageUtil;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.*;
import com.gic.haoban.manage.service.service.*;
import com.gic.mq.sdk.GicMQClient;
import com.gic.thirdparty.api.dto.PicUploadResDTO;
import com.gic.thirdparty.api.service.QQCloudPicService;
import com.gic.wechat.api.dto.qywx.ItemDTO;
import com.gic.wechat.api.dto.qywx.QywxXcxSendMessageDTO;
import com.gic.wechat.api.service.qywx.QywxDepartmentApiService;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.google.common.collect.Sets;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.net.URL;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 接收企业微信或者gic推送消息进行处理
 * Created by tgs on 2020/2/20.
 */
@Service("messageApiService")
public class MessageApiServiceImpl implements MessageApiService {
    private static final Logger log = LogManager.getLogger(MessageApiServiceImpl.class);
    private final String CONTACT_APP = "contactSuiteId";
    @Autowired
    private DepartmentService departmentService;
    @Autowired
    private DepartmentApiService departmentApiService;
    @Autowired
    private WxEnterpriseService enterpriseService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private StaffDepartmentRelatedService staffDepartmentRelatedService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private Config config;
    @Autowired
    private QywxDepartmentApiService qywxDepartmentApiService;

    @Autowired
    private QQCloudPicService qqCloudPicService;

    @Autowired
    private InnerApiService innerApiService;

    @Autowired
    private StaffClerkRelationApiService staffClerkRelationApiService;

    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;

    @Autowired
    private WxEnterpriseService wxEnterpriseService;

    @Autowired
    private StaffDepartmentRelatedApiService staffDepartmentRelatedApiService;

    @Autowired
    private WxEnterpriseRelatedService wxEnterpriseRelatedService;
    @Autowired
    private ClerkService clerkService;

    @Autowired
    private SecretSettingService secretSettingService;

    @Override
    public void gicMessageReceive(GicMessageDTO gicMessageDTO) {
        if (true) {
            log.info("好办3.0去除消息监听");
            return;
        }
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            log.info("发送消息到dealGicMessageMq，{}", JSON.toJSONString(gicMessageDTO));
            clientInstance.sendMessage("dealGicMessageMq", JSON.toJSONString(gicMessageDTO));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void wxMessageReceive(QywxCallBackDTO qywxCallBackDTO) {
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        // 处理定制-目前鸿星尔克用
        boolean checkErkeDeal = checkErkeDeal(qywxCallBackDTO);
        if (!checkErkeDeal) {
            log.info("不符合鸿星尔克需求,返回");
            return;
        }
        try {
            // 放开限制
            // if (config.getSuiteId().equals(qywxCallBackDTO.getSuiteId()) || CONTACT_APP.equals(qywxCallBackDTO.getSuiteId())) {

            // }
            log.info("发送消息到dealWxMessageMq，{}", JSON.toJSONString(qywxCallBackDTO));
            clientInstance.sendMessage("dealWxMessageMq", JSON.toJSONString(qywxCallBackDTO));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void dealWxMessage(String param) {
        QywxCallBackDTO dto = JSON.parseObject(param, QywxCallBackDTO.class);
        log.info("【处理微信回调】dto={}", JSON.toJSONString(param));
        TabHaobanWxEnterprise enterprise = enterpriseService.getEnterpriseBycorpId(dto.getAuthCorpId());
        if (Objects.isNull(enterprise)) {
            log.info("通过corpId:【{}】未查询到授权企业,无法处理", dto.getAuthCorpId());
            return;
        }
        // 校验适用企业-定制
        boolean suitWxEnterpriseIid = checkSuitEnterpriseId(enterprise.getWxEnterpriseId());
        if (!suitWxEnterpriseIid) {
            log.info("微信企业id:{}, 不适用", enterprise.getWxEnterpriseId());
            return;
        }
        List<TabHaobanWxEnterpriseRelated> relatedList = wxEnterpriseRelatedService.getByWxEnterpriseId(enterprise.getWxEnterpriseId());
        if (CollectionUtils.isEmpty(relatedList)) {
            log.info("未查询到企业关联企业 wxEnt");
            return;
        }
        ServiceResponse response = staffApiService.getWxSaveNew(dto.getUserid(), enterprise.getWxEnterpriseId());
        if (response.getCode() != 1) {
            log.info("添加企业员工失败:{}", response.getMessage());
            return;
        }
        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(enterprise.getWxEnterpriseId(), SecretTypeEnum.HAOBAN_HELP.getVal());
        if (null == secretSetting || secretSetting.getCheckFlag() == 0) {
            log.info("没有配置secret：{}", JSONObject.toJSONString(enterprise));
            return;
        }
        // 这里将员工姓名改一下, 代开发返回的 id 和 name 一样
        if (Objects.equals(dto.getUserid(), dto.getUserName())) {
            dto.setUserName((String) response.getResult());
        }
        // 取第一个企业的id
        doDealUser(dto, relatedList.get(0).getEnterpriseId(), secretSetting.getSecretVal());

        // 以下暂时不会用到
//        if (dto != null) {
//            String corpId = dto.getAuthCorpId();
//            TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(corpId);
//            if (enterprise != null) {
//                if (dto.getChangeType().endsWith("party")) {
//                    this.dealDepartment(dto);
//                } else {
//                    this.dealUser(dto);
//                }
//            }
//        }
    }

    /**
     * desc: 鸿星尔克定制需求校验-后期可扩展为配置形式
     *
     * @param dto dto
     * @return : {@link boolean}
     * @author : YongEn
     * @date : 2022/1/25
     */
    private boolean checkErkeDeal(QywxCallBackDTO dto) {
        if (Objects.isNull(dto) || StringUtils.isEmpty(dto.getUserid())) {
            return false;
        }
        if (!"create_user".equals(dto.getChangeType())) {
            log.info("只处理新增用户, 非 create_user 类型不做处理 当前传入类型:{}", dto.getChangeType());
            return false;
        }
        String userid = dto.getUserid();
        // 只处理5开头的-不是5开头不同步
        return userid.startsWith("5");
    }

    private boolean checkSuitEnterpriseId(String wxEnterpriseId) {
        Set<String> wxSet = Sets.newHashSet(
                // 润辰
                "ca66a01b79474c40b3e7c7f93daf1a3b",
                // 尊兴
                "c0c2186a947440aa8c0afbdd869ec043",
                "ac451a350eed4a96958dd5c7377f8bd9",
                // 鸿星尔克
                "e034e2bd5d0b44d3bf553c9f8204094c");
        return wxSet.contains(wxEnterpriseId);
    }

    private void doDealUser(QywxCallBackDTO dto, String enterpriseId, String secret) {
        ClerkQwDTO clerkQwDTO = new ClerkQwDTO();
        clerkQwDTO.setEnterpriseId(enterpriseId);
        String mainDeptId = dto.getMainDepartment();
        List<com.gic.wechat.api.dto.qywx.DepartmentDTO> departments = qywxDepartmentApiService.listSelfDepartment(dto.getAuthCorpId(), secret, Integer.valueOf(mainDeptId));
        if (CollectionUtils.isEmpty(departments)) {
            log.info("通过主部门查询部门列表为空 mainDeptId:{}", mainDeptId);
            return;
        }
        Map<Integer, com.gic.wechat.api.dto.qywx.DepartmentDTO> departmentIdMap = departments.stream()
                .collect(Collectors.toMap(com.gic.wechat.api.dto.qywx.DepartmentDTO::getId, Function.identity(), (a, b) -> a));
        com.gic.wechat.api.dto.qywx.DepartmentDTO mainDept = departmentIdMap.get(Integer.valueOf(mainDeptId));
        clerkQwDTO.setStoreName(mainDept.getName());
        clerkQwDTO.setClerkCode(dto.getUserid());
        clerkQwDTO.setClerkName(dto.getUserName());
        clerkQwDTO.setClerkType(dto.getIsLeaderInDept()[0]);
        clerkQwDTO.setClerkGener(dto.getGender());
        clerkQwDTO.setPhoneNumber(dto.getMobile());
        clerkQwDTO.setNationcode(dto.getTelephone());
        // 调用换导购接口逻辑
        clerkService.syncClerkForQw(clerkQwDTO);
    }

    private void dealDepartment(QywxCallBackDTO qywxCallBackDTO) {
        DepartmentDTO departmentDTO = new DepartmentDTO();
        departmentDTO.setDepartmentName(qywxCallBackDTO.getDepartmentName());
        departmentDTO.setIsStore(0);
        //departmentDTO.setRelatedId(qywxCallBackDTO.getId()+"");
        TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(qywxCallBackDTO.getAuthCorpId());
        TabHaobanDepartment parentDepartment = new TabHaobanDepartment();
        if (qywxCallBackDTO.getParentId() != null) {
            parentDepartment = this.departmentService.getByWxId(qywxCallBackDTO.getParentId() + "", enterprise.getWxEnterpriseId());
        } else {
            TabHaobanDepartment oldDepartment = this.departmentService.getByWxId(qywxCallBackDTO.getId() + "", enterprise.getWxEnterpriseId());
            if (oldDepartment != null) {
                parentDepartment = departmentService.selectById(oldDepartment.getParentDepartmentId());
            }
        }
        if (!qywxCallBackDTO.getChangeType().equals(WxEditType.DELETEDEPART.getCode())) {
            if (parentDepartment == null) {
                log.info("微信同步父部门不存在");
                return;
            }
            departmentDTO.setParentDepartmentId(parentDepartment.getDepartmentId());
            departmentDTO.setChainId(parentDepartment.getChainId() + Constant.ID_SEPARATOR + parentDepartment.getDepartmentId());
            departmentDTO.setChainName(parentDepartment.getChainName() + Constant.NAME_SEPARATOR + parentDepartment.getChainName());
            departmentDTO.setLevel(parentDepartment.getLevel() + 1);
        }
        departmentDTO.setWxEnterpriseId(enterprise.getWxEnterpriseId());
        departmentDTO.setWxDepartmentId(qywxCallBackDTO.getId() + "");
        if (qywxCallBackDTO.getParentId() != null) {
            List<com.gic.wechat.api.dto.qywx.DepartmentDTO> list = this.qywxDepartmentApiService.listDepartment(qywxCallBackDTO.getAuthCorpId(), qywxCallBackDTO.getSuiteId(), qywxCallBackDTO.getParentId());
            if (list != null) {
                for (com.gic.wechat.api.dto.qywx.DepartmentDTO departmentDTO2 : list) {
                    if (departmentDTO2.getId().equals(departmentDTO.getDepartmentId())) {
                        departmentDTO.setSort(qywxCallBackDTO.getOrder());
                    } else {
                        TabHaobanDepartment tab = this.departmentService.getByWxId(departmentDTO2.getId() + "", enterprise.getWxEnterpriseId());
                        if (tab != null) {
                            tab.setSort(departmentDTO2.getOrder());
                            this.departmentService.edit(EntityUtil.changeEntityByJSON(DepartmentDTO.class, tab));
                        }
                    }
                }
            }
        }
        if (qywxCallBackDTO.getChangeType().equals(WxEditType.ADDDEPART.getCode())) {
            String response = this.departmentService.add(departmentDTO);
            log.info("微信同步新增部门：{}", JSON.toJSONString(response));
        } else if (qywxCallBackDTO.getChangeType().equals(WxEditType.UPDATEDEPART.getCode())) {
            TabHaobanDepartment department = this.departmentService.getByWxId(qywxCallBackDTO.getId() + "", enterprise.getWxEnterpriseId());
            if (department == null) {
                String response = this.departmentService.add(departmentDTO);
                log.info("微信同步新增部门：{}", JSON.toJSONString(response));
            } else {
                departmentDTO.setIsStore(department.getIsStore());
                departmentDTO.setDepartmentId(department.getDepartmentId());
                this.departmentService.edit(departmentDTO);
                log.info("企业微信更新同步完成");
            }
        } else {
            TabHaobanDepartment department = this.departmentService.getByWxId(qywxCallBackDTO.getId() + "", enterprise.getWxEnterpriseId());
            if (department == null) {
                log.info("微信同步删除部门不存在");
            } else {
                if (StringUtils.isBlank(department.getRelatedId())) {
                    this.departmentService.del(department.getDepartmentId());
                } else {
                    this.departmentApiService.wxCallBackrecycle(department.getDepartmentId());
                }
                log.info("企业微信删除同步完成");
            }
        }
    }

    private void dealUser(QywxCallBackDTO qywxCallBackDTO) {
        TabHaobanStaff staff = new TabHaobanStaff();
        TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(qywxCallBackDTO.getAuthCorpId());
        if (enterprise != null) {
            staff.setWxEnterpriseId(enterprise.getWxEnterpriseId());
            //staff.setActiveFlag(0);
            staff.setPhoneNumber(qywxCallBackDTO.getMobile());
            staff.setNickName(qywxCallBackDTO.getAlias());
            staff.setSex(qywxCallBackDTO.getGender());
            staff.setStaffName(qywxCallBackDTO.getUserName());
            staff.setWxUserId(qywxCallBackDTO.getUserid());
            staff.setPostion(qywxCallBackDTO.getPosition());
            if (StringUtils.isNotBlank(qywxCallBackDTO.getAvatar())) {
                staff.setHeadImg(changeHeaderImageUrl(qywxCallBackDTO.getAvatar()));
            }
            staff.setNationCode("86");
            //激活状态
            if (qywxCallBackDTO.getStatus() != null && qywxCallBackDTO.getStatus() == 1) {
                staff.setActiveFlag(1);
            }
            if (qywxCallBackDTO.getChangeType().equals(WxEditType.ADDUSER.getCode())) {
                if (staff.getActiveFlag() == null) {
                    staff.setActiveFlag(0);
                }
                this.addUser(qywxCallBackDTO, staff);
            } else if (qywxCallBackDTO.getChangeType().equals(WxEditType.UPDATEUSER.getCode())) {
                TabHaobanStaff oldStaff = this.staffService.selectByUserIdAndEnterpriseId(qywxCallBackDTO.getUserid(), enterprise.getWxEnterpriseId());
                if (oldStaff != null) {
                    staff.setWxUserId(qywxCallBackDTO.getNewUserid());
                    staff.setStaffId(oldStaff.getStaffId());
                    log.info("【微信回调更新】staff={}", JSON.toJSONString(staff));
                    if (StringUtils.isBlank(staff.getPhoneNumber())) {
                        staff.setPhoneNumber(oldStaff.getPhoneNumber());
                    }
                    if (StringUtils.isBlank(staff.getNationCode())) {
                        staff.setNationCode(oldStaff.getNationCode());
                    }
                    if (StringUtils.isBlank(staff.getStaffName())) {
                        staff.setStaffName(oldStaff.getStaffName());
                    }
                    if (StringUtils.isBlank(staff.getPostion())) {
                        staff.setPostion(oldStaff.getPostion());
                    }
                    if (staff.getSex() == null) {
                        staff.setSex(oldStaff.getSex());
                    }
                    if (staff.getActiveFlag() == null) {
                        staff.setActiveFlag(oldStaff.getActiveFlag());
                    }
                    if (StringUtils.isBlank(staff.getHeadImg())) {
                        staff.setHeadImg(oldStaff.getHeadImg());
                    }
//                    this.staffService.updateByPrimaryKey(staff);
//                    this.staffDepartmentRelatedService.delByUserid(qywxCallBackDTO.getUserid());
//                    this.addUserDepart(qywxCallBackDTO, oldStaff.getStaffId(), enterprise.getWxEnterpriseId());
                    String[] departArr = qywxCallBackDTO.getDepartment();
                    String departmentIds = "";
                    //部门修改了
                    if (departArr != null) {
                        for (String s : departArr) {
                            TabHaobanDepartment department = this.departmentService.getByWxId(s, enterprise.getWxEnterpriseId());
                            if (department == null) {
                                log.info("部门不存在");
                                continue;
                            }
                            departmentIds += department.getDepartmentId() + ",";
                        }
                        departmentIds = departmentIds.substring(0, departmentIds.length() - 1);
                    } else {
                        //部门没有修改
                        List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staff.getStaffId());
                        for (TabHaobanStaffDepartmentRelated tabHaobanStaffDepartmentRelated : list) {
                            departmentIds += tabHaobanStaffDepartmentRelated.getDepartmentId() + ",";
                        }
                        departmentIds = departmentIds.substring(0, departmentIds.length() - 1);
                    }
                    StaffDTO staffDTO = EntityUtil.changeEntityByJSON(StaffDTO.class, staff);
                    staffDTO.setWeixinPush(true);
                    staffApiService.staffEdit(staffDTO, departmentIds);

                } else {
                    if (staff.getActiveFlag() == null) {
                        staff.setActiveFlag(0);
                    }
                    this.addUser(qywxCallBackDTO, staff);
                }
            } else {
                TabHaobanStaff oldStaff = this.staffService.selectByUserIdAndEnterpriseId(qywxCallBackDTO.getUserid(), enterprise.getWxEnterpriseId());
                //List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staff.getStaffId());
                if (oldStaff != null) {
                    List<TabHaobanStaffDepartmentRelated> relatedList = staffDepartmentRelatedService.listStaffDepartmentByStaffId(oldStaff.getStaffId());
                    TabHaobanStaffDepartmentRelated staffRelated = relatedList.get(0);
                    if (StringUtils.isNotBlank(staffRelated.getClerkCode())) {
                        staffApiService.cleanGicClerk(staffRelated.getClerkCode(), staffRelated.getDepartmentId());
                    }
                    innerApiService.delCardByStaffId(oldStaff.getStaffId());
                    this.staffDepartmentRelatedService.del(EntityUtil.changeEntityByJSON(StaffDepartmentRelatedDTO.class, staffRelated));
                    this.staffService.delByuserid(oldStaff.getStaffId());
                } else {
                    log.info("微信同步删除的门店不存在，{}", qywxCallBackDTO.getUserid());
                }
            }
        } else {
            log.info("授权企业不存在，{}", qywxCallBackDTO.getAuthCorpId());
        }
    }


    private void addUser(QywxCallBackDTO qywxCallBackDTO, TabHaobanStaff staff) {
        TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(qywxCallBackDTO.getAuthCorpId());
        StaffDTO staffDTO = staffApiService.selectByUserIdAndEnterpriseId(staff.getWxUserId(), enterprise.getWxEnterpriseId());
        String[] departArr = qywxCallBackDTO.getDepartment();
        String departmentIds = "";
        for (String s : departArr) {
            TabHaobanDepartment department = this.departmentService.getByWxId(s, enterprise.getWxEnterpriseId());
            if (department == null) {
                log.info("部门不存在");
                continue;
            }
            departmentIds += department.getDepartmentId() + ",";
        }
        departmentIds = departmentIds.substring(0, departmentIds.length() - 1);
        if (staffDTO == null) {
            staffApiService.wxGetAdd(staff.getWxUserId(), enterprise.getWxEnterpriseId());
        } else {
            List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staff.getStaffId());
            for (TabHaobanStaffDepartmentRelated tabHaobanStaffDepartmentRelated : list) {
                if (!departmentIds.contains(tabHaobanStaffDepartmentRelated.getDepartmentId())) {
                    departmentIds += tabHaobanStaffDepartmentRelated.getDepartmentId() + ",";
                }
                if (departmentIds.endsWith(",")) {
                    departmentIds = departmentIds.substring(0, departmentIds.length() - 1);
                }
            }
            StaffDTO staffUpdate = EntityUtil.changeEntityByJSON(StaffDTO.class, staff);
            staffUpdate.setWeixinPush(true);
            staffApiService.staffEdit(staffUpdate, departmentIds);
        }
//        String add = this.staffService.add(staff);
//        log.info("新增用户返回结果：{}", add);
//        this.addUserDepart(qywxCallBackDTO, add, staff.getWxEnterpriseId());
    }

    private void addUserDepart(QywxCallBackDTO qywxCallBackDTO, String staffId, String wxEnterpriseId) {
        String[] departArr = qywxCallBackDTO.getDepartment();
        if (departArr != null) {
            for (String s : departArr) {
                TabHaobanDepartment department = this.departmentService.getByWxId(s, wxEnterpriseId);
                if (department == null) {
                    log.info("部门不存在");
                    continue;
                }
                StaffDepartmentRelatedDTO related = new StaffDepartmentRelatedDTO();
                related.setWxEnterpriseId(wxEnterpriseId);
                related.setWxUserId(qywxCallBackDTO.getUserid());
                related.setClerkCode(qywxCallBackDTO.getUserid());
                related.setDepartmentId(department.getDepartmentId());
                related.setPhoneNumber(qywxCallBackDTO.getMobile());
                //related.setStaffDepartmentRelatedId(staffId);
                related.setStaffId(staffId);
                related.setStatusFlag(1);
                String add1 = this.staffDepartmentRelatedService.add(related);
                log.info("新增用户部门关联关系：{}", add1);
            }
        }
    }

    private String changeHeaderImageUrl(String headImgUrl) {
        try {
            InputStream in = new URL(headImgUrl).openStream();
            byte[] data = IOUtils.toByteArray(in);
            PicUploadResDTO uploadPic = qqCloudPicService.uploadPic(GlobalVar.ctxPropertiesMap.get(GlobalInfo.QQPIC_KEY_ENTERPRISE), data);
            log.info("腾讯云万象优图返回" + JSON.toJSONString(uploadPic));
            return uploadPic.downloadUrl;
        } catch (Exception e) {
            log.info("上传腾讯云万象优图返回失败了：", e);
            return headImgUrl;
        }
    }

    /**
     * 通过关联的父级ID链查询父级部门
     *
     * @param parentChain
     * @return
     */
    private DepartmentDTO getParentDepartment(String parentChain, int dataType, String wxEnterpriseId) {
        if (StringUtils.isNotBlank(parentChain)) {
            String[] arr = parentChain.split("_");
            int len = arr.length;
            int startIndex = len - 2;
            if (dataType == GicDataType.STORE.getCode() || dataType == GicDataType.DEPARTMENT.getCode()) {
                startIndex = len - 1;
            }
            for (int i = startIndex; i >= 0 && i < len; i--) {
                DepartmentDTO department = this.departmentService.getByRelatedIdNotInRecycle(arr[i]);
                log.info("当前节点id：{}, {}", arr[i], JSON.toJSONString(department));
                if (department != null) {
                    return department;
                }
            }
        }
        //DepartmentDTO departmentDTO = this.departmentService.getRootByEnterpriseId(wxEnterpriseId);
        return null;
    }

    @Override
    public Boolean sendFaceMessage(String clerkId, String memberId, String memberName, String arrivalTime, String storeId) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("memberId", memberId);
        jsonObject.put("storeId", storeId);
        String data = jsonObject.toJSONString();
        String pageUrl = staffDepartmentRelatedApiService.getPageUrl(AppPageType.FACE_ARRIVAL.getCode(), data);
        StaffClerkRelationDTO relation = staffClerkRelationApiService.getByClerkId(clerkId);
        if (relation == null) {
            log.info("clerkId未绑定:clerkId={}", clerkId);
            return false;
        }
        WxEnterpriseDTO wxEnterprise = wxEnterpriseService.selectById(relation.getWxEnterpriseId());
        if (wxEnterprise == null) {
            log.info("企业为空");
            return false;
        }
        StaffDTO staffDTO = staffApiService.selectById(relation.getStaffId());
        String corpid = wxEnterprise.getCorpid();
        String wxUserId = "";
        if (corpid.length() > 20) {
            wxUserId = staffDTO.getWxOpenUseId();
        } else {
            wxUserId = staffDTO.getWxUserId();
        }
        QywxXcxSendMessageDTO messageDTO = new QywxXcxSendMessageDTO();
        Map<String, String> map = new HashMap<>();
        map.put("事件", "人脸匹配");
        map.put("会员", memberName);
        map.put("到店时间", arrivalTime);
        List<ItemDTO> items = getItemsList(map);
        ArrayList<String> list = new ArrayList<>();
        list.add(wxUserId);
        messageDTO.setAppid(config.getAppid());
        messageDTO.setUserIds(list);
        messageDTO.setPage(pageUrl);
        messageDTO.setTitle("我的顾客通知");
        messageDTO.setItems(items);
        return qywxSuiteApiService.sendMessage(corpid, config.getWxSuiteid(), messageDTO);
    }

    private List<ItemDTO> getItemsList(Map<String, String> map) {
        List<ItemDTO> items = new ArrayList();
        map.keySet().forEach(key -> {
            String value = map.get(key);
            ItemDTO dto = new ItemDTO();
            dto.setKey(key);
            dto.setValue(value);
            items.add(dto);
        });
        return items;
    }

    @Override
    public Boolean sendPerfectRemarkMessage(String clerkId, String memberId, String memberName, String customerTime, String orderId, String storeId) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("memberId", memberId);
        jsonObject.put("storeId", storeId);
        jsonObject.put("currentTab", 1);
//        String data = jsonObject.toJSONString();
//        String pageUrl = staffDepartmentRelatedApiService.getPageUrl(AppPageType.PERFECT_TAG.getCode(), data);
        StaffClerkRelationDTO relation = staffClerkRelationApiService.getByClerkId(clerkId);
        if (relation == null) {
            log.info("clerkId未绑定:clerkId={}", clerkId);
            return false;
        }
        WxEnterpriseDTO wxEnterprise = wxEnterpriseService.selectById(relation.getWxEnterpriseId());
        if (wxEnterprise == null) {
            log.info("企业为空");
            return false;
        }
        StaffDTO staffDTO = staffApiService.selectById(relation.getStaffId());
        if (staffDTO == null) {
            return false;
        }
//        String corpId = wxEnterprise.getCorpid();
//        String wxUserId = "";
//        if (corpId.length() > 20) {
//            wxUserId = staffDTO.getWxOpenUseId();
//        } else {
//            wxUserId = staffDTO.getWxUserId();
//        }
//        QywxXcxSendMessageDTO messageDTO = new QywxXcxSendMessageDTO();
//        Map<String, String> map = new HashMap<>();
//        map.put("事件", "会员完善标签");
//        map.put("会员", memberName);
//        map.put("消费订单", orderId);
//        map.put("消费时间", customerTime);
//        List<ItemDTO> items = getItemsList(map);
//        ArrayList<String> list = new ArrayList<>();
//        list.add(wxUserId);
//        messageDTO.setAppid(config.getAppid());
//        messageDTO.setUserIds(list);
//        messageDTO.setPage(pageUrl);
//        messageDTO.setTitle("我的顾客通知");
//        messageDTO.setItems(items);
//
//        return qywxSuiteApiService.sendMessage(wxEnterprise.getCorpid(), config.getWxSuiteid(), messageDTO);

        Map<String, String> params = new HashMap<>();
        params.put("memberName", memberName);
        params.put("orderId", orderId);
        params.put("customerTime", customerTime);

        NoticeMessageUtil.sendNoticeMessage(relation.getEnterpriseId(), relation.getClerkId(), NoticeMessageTypeEnum.MEMBER_FACE.getType(), null, params, jsonObject);
        return true;

    }

    @Override
    public void sendWxMessage(String param) {
        log.info("发送企业微信消息：{}", param);
        if (StringUtils.isBlank(param)) {
            log.info("param参数为空");
            return;
        }
        JSONObject jsonObject = JSON.parseObject(param);
        String wxEnterpriseId = jsonObject.getString("wxEnterpriseId");
        String wxUserId = jsonObject.getString("wxUserId");
        String pageUrl = jsonObject.getString("pageUrl");
        String title = jsonObject.getString("title");
        String content = jsonObject.getString("content");
        String memberName = jsonObject.getString("memberName");
        WxEnterpriseDTO wxEnterprise = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterprise == null) {
            log.info("企业为空");
            return;
        }
        QywxXcxSendMessageDTO messageDTO = new QywxXcxSendMessageDTO();
        //后续map加判断可加参数
        Map<String, String> map = new HashMap<>();
        map.put("接收事件", content);
        map.put("接收人", memberName);
        List<ItemDTO> items = getItemsList(map);
        List<String> userList = new ArrayList<>();
        userList.add(wxUserId);
        messageDTO.setAppid(config.getAppid());
        messageDTO.setUserIds(userList);
        messageDTO.setPage(pageUrl);
        messageDTO.setTitle(title);
        messageDTO.setItems(items);
        qywxSuiteApiService.sendMessage(wxEnterprise.getCorpid(), config.getWxSuiteid(), messageDTO);
    }

    @Override
    public String getNewWxUserId(String wxEnterpriseId, String wxUserId) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        String corpid = wxEnterpriseDTO.getCorpid();

        TabHaobanStaff tabHaobanStaff = staffService.selectByUserIdAndEnterpriseId(wxUserId, wxEnterpriseId);

        if (corpid.length() > 20) {
            wxUserId = tabHaobanStaff.getWxOpenUseId();
        } else {
            wxUserId = tabHaobanStaff.getWxUserId();
        }
        return wxUserId;
    }


}
