package com.gic.haoban.manage.service.task;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.DealParamMqDTO;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.DealSyncOperationApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.service.dao.mapper.WxApplicationMapper;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.entity.TabHaobanSyncTask;
import com.gic.haoban.manage.service.entity.TabHaobanWxApplication;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.dto.qywx.QywxNewUseridDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.collections.CollectionUtils;
import org.redisson.api.RRateLimiter;
import org.redisson.api.RateIntervalUnit;
import org.redisson.api.RateType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 企微通讯录同步
 * Created 2020/4/13.
 *
 * @author hua
 */
@Service
public class QywxClerkSyncOperation implements BaseSyncOperation {
    private static final Logger logger = LoggerFactory.getLogger(QywxClerkSyncOperation.class);

    @Autowired
    private PreDealService preDealService;
    @Autowired
    private SyncTaskService syncTaskService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private DealSyncOperationApiService dealSyncOperationApiService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;
    @Autowired
    private WxApplicationMapper wxApplicationMapper;

    @Override
    public void dealSingleByMq(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
        logger.info("企微通讯录成员同步处理：{}", JSONObject.toJSONString(dealParamMqDTO));
        boolean dealFlag = true;
        String reason = "";
        //处理clerk
        try {
            ServiceResponse response = staffApiService.getWxSaveNew(dealParamMqDTO.getData(), dataPre.getWxEnterpriseId());
            if (response.getCode() != 1) {
                dealFlag = false;
                reason = response.getMessage();
            }
        } catch (Exception e) {
            logger.info("企微通讯录成员同步失败：{}", e.getMessage(), e);
            reason = "同步异常:";
            dealFlag = false;
        } finally {
            if (!dealFlag) {
                dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), reason);
            } else {
                dealSuccess(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), dataPre.getWxEnterpriseId());
            }
        }
    }

    @Override
    public void dealException(String taskId, String dataId, String enterpriseId, String reason) {
        logger.info("企微通讯录成员处理失败：t:{},d:{}", taskId, dataId);
        boolean b = preDealService.updateStatusByDataId(taskId, dataId, PreDealTypeEnum.clerk.getVal(), PreDealStatusEnum.exception.getVal(), reason);
        if (!b) {
            logger.info("企微通讯录成员处理异常:t:{}, d:{}", taskId, dataId);
            return;
        }
        checkDepartmentTask(taskId);

    }

    @Override
    public void dealSuccess(String taskId, String dataId, String enterpriseId, String wxEnterpriseId) {
        logger.info("企微通讯录成员处理成功：t:{},d:{}", taskId, dataId);
        boolean b = preDealService.updateStatusByDataId(taskId, dataId, PreDealTypeEnum.clerk.getVal(), PreDealStatusEnum.computed.getVal(), "成功");
        if (!b) {
            logger.info("企微通讯录成员成功修改错误！{}", dataId);
            return;
        }
        checkDepartmentTask(taskId);
    }

    @Override
    public void checkDepartmentTask(String taskId) {
        boolean b = preDealService.checkTask(taskId, PreDealTypeEnum.clerk.getVal());
        if (!b) {
            return;
        }
        logger.info("企微通讯录成员成功:{}", taskId);
        String key = "haoban_sync_clerk_task_" + taskId;
        RedisUtil.lock(key, 3L);
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        String wxEnterpriseId = syncTask.getWxEnterpriseId();
        //同步成功 进入门店处理
        if (syncTask.getStatusFlag().equals(SyncTaskStatusEnum.clerk_sync.getVal())) {
            int errCount = preDealService.countByTaskId(taskId, -1, PreDealStatusEnum.exception.getVal());
            if (errCount > 0) {
                syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.exception_compute.getVal());
                dealSyncOperationApiService.unlockTask(wxEnterpriseId);
            } else {
                syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.compute.getVal());
                dealSyncOperationApiService.unlockTask(wxEnterpriseId);
                dealSyncOperationApiService.cleanDiffrence(wxEnterpriseId, taskId);
                WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(wxEnterpriseId);
                String corpid = wxEnterpriseDTO.getCorpid();
                Integer externalFlag = wxEnterpriseDTO.getExternalFlag();
                TabHaobanWxApplication tabHaobanWxApplication = wxApplicationMapper.selectByCorpId(corpid, null);
                logger.info("同步通讯录完成，批量处理wx_user_id，corpid：{}", corpid);
                if (corpid.length() > 20 || externalFlag == 1) {
                    // 限流器
                    RRateLimiter rateLimiter = RedisUtil.getRedisClient().getRateLimiter("haoban:manage3:updateWxOpenUserId:limit");
                    // 每1秒产生5个令牌
                    rateLimiter.trySetRate(RateType.OVERALL, 5, 1, RateIntervalUnit.SECONDS);
                    int pageNum = 1;
                    while (true) {
                        // 获取令牌
                        rateLimiter.acquire();
                        //企微转换最多支持1000个
                        //批量处理wx_user_id to wx_open_user_id
                        Page<StaffDTO> staffDTOPageInfo = staffApiService.listUserIdByWxEnterpriseId(wxEnterpriseId, pageNum, 1000);
                        List<StaffDTO> staffDTOS = staffDTOPageInfo.getResult();
                        if (CollectionUtils.isEmpty(staffDTOS)) {
                            logger.info("转换完成，为最后一个");
                            break;
                        }
                        pageNum++;
                        logger.info("useridToOpenuserid请求：corpid：{}", corpid);
                        List<QywxNewUseridDTO> qywxNewUseridDTOS = qywxUserApiService.useridToOpenuserid(corpid, tabHaobanWxApplication.getSiteId(), staffDTOS.stream().map(StaffDTO::getWxUserId).collect(Collectors.toList()));
                        if (CollectionUtils.isNotEmpty(qywxNewUseridDTOS)) {
                            for (QywxNewUseridDTO qywxNewUseridDTO : qywxNewUseridDTOS) {
                                staffApiService.updateOpenUserIdsByUserId(wxEnterpriseId, qywxNewUseridDTO.getUserid(), qywxNewUseridDTO.getOpen_userid());
                            }
                        }
                    }
                }
            }
        }
        RedisUtil.unlock(key);
    }


    private Page<StaffDTO> pageStaffListByWxEnterpriseId(String wxEnterpriseId, Integer pageNum, Integer pageSize) {
        return staffApiService.listUserIdByWxEnterpriseId(wxEnterpriseId, pageNum, pageSize);
    }


}
